package lib.ui.reactive;

import util.ObjMerge;
import lib.Pos;
import lib.Rect;
import lib.Vec.Vec2;

using tink.CoreApi;

abstract class UIElement {
	/**
		Render the element inside the bounds. `offset` is the offset to the parents position
		and can be used to calculate the absolute position of element.
		Just save `offset` and pass it to the children.
	**/
	abstract public function render(bounds:Vec2<Int>, offset: Pos):Canvas;
	public var changed(default, null):Signal<Noise>;
	private final changedTrigger:SignalTrigger<Noise> = Signal.trigger();
	public final eventListner:UIEvents = {};

	public function new(events: UIEvents = null) {
		changed = changedTrigger.asSignal();
		if (events != null){
			this.eventListner = events;
		}
	}

	public static function getElementInMap(pos: Pos,elementMap: Map<Rect,UIElement>):Null<UIElement>{
		for (k => v in elementMap){
			if (k.isInside(pos)){
				return v;
			}
		}

		return null;
	};

	public static function applyPaddignAndMargin(canvas:Canvas,padding: Dimensions, margin: Dimensions):Canvas{
		var passing = ObjMerge.merge(padding, {top: 0, left: 0, bottom: 0, right: 0});
		var margin = ObjMerge.merge(margin, {top: 0, left: 0, bottom: 0, right: 0});
		var rtn = new Canvas();

		rtn.combine(canvas,{x:margin.left + padding.left,y: margin.top + padding.top});

		return rtn;
	}
}
